# Copyright 2017, 2018, 2019 Cumulus Networks, Inc.  All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt

from __future__ import absolute_import
import subprocess
import yaml
from threading import Timer

from netq_lib.orm.redisdb.models import ProcNetDevInterface

class ReturnGroup:
    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)

def runCommand(cmd):
    try:
        proc = subprocess.Popen(cmd.split(), stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        timer = Timer(2, proc.kill)
        try:
            timer.start()
            (out, err) = proc.communicate()
        finally:
            timer.cancel()
        if proc.returncode != 0:
            return ReturnGroup(success=False, op="")
        else:
            return ReturnGroup(success=True, op=out)
    except OSError:
        return ReturnGroup(success=False, op="")

def process_proc_net_dev(output, svc_failed=False, **kwargs):
    """
    Parse /proc/net/dev counters
    """

    command = "ethtool -S "
    with open('/proc/net/dev', 'r') as fh:
        '''
        /proc/net/dev has the following format

Inter-|   Receive                                                |  Transmit
 face |bytes    packets errs drop fifo frame compressed multicast|bytes    packets errs drop fifo colls carrier compressed
 swp27:       0       0    0    0    0     0          0         0        0       0    0    0    0     0       0          0
 swp19:       0       0    0    0    0     0          0         0        0       0    0    0    0     0       0          0
 swp50:       0       0    0    0    0     0          0         0        0       0    0    0    0     0       0          0
 swp42:       0       0    0    0    0     0          0         0        0       0    0    0    0     0       0          0
 swp10:       0       0    0    0    0     0          0         0        0       0    0    0    0     0       0          0
 swp34:       0       0    0    0    0     0          0         0        0       0    0    0    0     0       0          0
  swp7: 14386332   36183    0 4459    0     0          0     13319 62337842   81001    0  454    0     0       0          0
 swp26:       0       0    0    0    0     0          0         0        0       0    0    0    0     0       0          0
        '''
        for line in fh.readlines()[2:]:
            (ifname,
             rx_bytes, rx_packets, rx_errs, rx_drop, rx_fifo, rx_frame, rx_compressed, rx_multicast,
             tx_bytes, tx_packets, tx_errs, tx_drop, tx_fifo, tx_colls, tx_carrier, tx_compressed) = line.strip().split()

            # chop the trailing :
            ifname = ifname[0:-1]

            # This is a virtual interface that is created as part of VRR, ignore it
            if ifname.endswith('-v0'):
                continue

            # fifo and compressed related counters are intentionally left out
            iface = ProcNetDevInterface()

            result = runCommand(command + ifname)
            if result.success:
                ethElems = {}
                try:
                    ethElems = yaml.safe_load(result.op)['NIC statistics']
                except yaml.YAMLError:
                    ethElems = {}

                if('HwIfInOctets' in ethElems and
                        'HwIfInUcastPkts' in ethElems and
                        'HwIfInBcastPkts' in ethElems and
                        'HwIfInMcastPkts' in ethElems and
                        'HwIfOutOctets' in ethElems and
                        'HwIfOutUcastPkts' in ethElems and
                        'HwIfOutMcastPkts' in ethElems and
                        'HwIfOutBcastPkts' in ethElems and
                        'HwIfInDiscards' in ethElems and
                        'HwIfInDot3LengthErrors' in ethElems and
                        'HwIfInErrors' in ethElems and
                        'HwIfOutDiscards' in ethElems and
                        'HwIfOutErrors' in ethElems):
                    iface = createIface(ifname,
                        rx_bytes=ethElems['HwIfInOctets'],
                        rx_drop=ethElems['HwIfInDiscards'],
                        rx_errs=ethElems['HwIfInDot3LengthErrors'] + ethElems['HwIfInErrors'],
                        rx_frame=ethElems['HwIfInUcastPkts'] + ethElems['HwIfInBcastPkts'] + ethElems['HwIfInMcastPkts'],
                        rx_multicast=ethElems['HwIfInMcastPkts'],
                        rx_packets=ethElems['HwIfInUcastPkts'] + ethElems['HwIfInBcastPkts'] + ethElems['HwIfInMcastPkts'],
                        tx_bytes=ethElems['HwIfOutOctets'],
                        tx_drop=ethElems['HwIfOutDiscards'],
                        tx_errs=ethElems['HwIfOutErrors'],
                        tx_packets=ethElems['HwIfOutUcastPkts'] + ethElems['HwIfOutMcastPkts'] + ethElems['HwIfOutBcastPkts'],
                        rx_broadcast=ethElems['HwIfInBcastPkts'],
                        tx_broadcast=ethElems['HwIfOutBcastPkts'],
                        tx_multicast=ethElems['HwIfOutMcastPkts'])

                else:
                    iface = createIface(ifname, rx_bytes=rx_bytes, rx_drop=rx_drop, rx_errs=rx_errs, rx_frame=rx_frame, rx_multicast=rx_multicast, rx_packets=rx_packets,
                                        tx_bytes=tx_bytes, tx_carrier=tx_carrier, tx_colls=tx_colls, tx_drop=tx_drop, tx_errs=tx_errs, tx_packets=tx_packets)
            else:
                iface = createIface(ifname, rx_bytes=rx_bytes, rx_drop=rx_drop, rx_errs=rx_errs, rx_frame=rx_frame, rx_multicast=rx_multicast, rx_packets=rx_packets,
                                        tx_bytes=tx_bytes, tx_carrier=tx_carrier, tx_colls=tx_colls, tx_drop=tx_drop, tx_errs=tx_errs, tx_packets=tx_packets)

            iface.save()

        iface_arr = []
        iface_arr.append(iface)

def createIface(ifname, rx_bytes=-1, rx_drop=-1, rx_errs=-1, rx_frame=-1, rx_multicast=-1, rx_packets=-1, tx_bytes=-1,
                tx_carrier=-1, tx_colls=-1, tx_drop=-1, tx_errs=-1, tx_packets=-1, rx_broadcast=-1, tx_broadcast=-1, tx_multicast=-1):
    # fifo and compressed related counters are intentionally left out
    iface = ProcNetDevInterface()
    iface.ifname = ifname
    iface.rx_bytes = rx_bytes
    iface.rx_drop = rx_drop
    iface.rx_errs = rx_errs
    iface.rx_frame = rx_frame
    iface.rx_multicast = rx_multicast
    iface.rx_packets = rx_packets
    iface.tx_bytes = tx_bytes
    iface.tx_carrier = tx_carrier
    iface.tx_colls = tx_colls
    iface.tx_drop = tx_drop
    iface.tx_errs = tx_errs
    iface.tx_packets = tx_packets
    iface.rx_broadcast = rx_broadcast
    iface.tx_broadcast = tx_broadcast
    iface.tx_multicast = tx_multicast
    return iface
